﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace RPGTest
{
    // キャラクタクラス（マップオブジェクトクラスを継承）
    class Character : MapObject
    {
        // ステータス情報
        public int HP;
        public int MaxHP;
        public int STR;
        public int DEF;
        public int Level;
        public int EXP;
        public string Status;
        
        // 持ち物情報
        public int Money;
        public Weapon Weapon;
        public Armor Armor;

        // アニメーション情報
        float frame = 0;
        public Direction Dir = Direction.Down;

        public Character()
        {
            origin = new Vector2(30, 60);
        }

        // キャラクタデータの初期化
        public void SetCharacterData(string name, string asset, int hp, int maxhp, int str, int def, int level, int exp, string status, int money, Weapon weapon, Armor armor, Vector2 position)
        {
            this.name = name;
            this.asset = asset;
            HP = hp;
            MaxHP = maxhp;
            STR = str;
            DEF = def;
            Level = level;
            EXP = exp;
            Status = status;
            Money = money;
            Weapon = weapon;
            Armor = armor;
            this.position = position;
        }

        // キャラクタの中心をメインパネルの中央に移動
        public void setCenter()
        {
            origin = new Vector2(image.Width / 2, image.Height / 2);
            position = new Vector2(300, 225);
        }

        // 移動
        public void Move(Direction dir)
        {
            Vector2 movePosition = position;

            // 仮移動
            switch (dir)
            {
                case Direction.Up:
                    movePosition.Y--;
                    break;
                case Direction.Down:
                    movePosition.Y++;
                    break;
                case Direction.Left:
                    movePosition.X--;
                    break;
                case Direction.Right:
                    movePosition.X++;
                    break;
            }

            // 移動制限チェック
            if (movePosition.X < 30) movePosition.X = 30;
            if (movePosition.X > 570) movePosition.X = 570;
            if (movePosition.Y < 60) movePosition.Y = 60;
            if (movePosition.Y > 420) movePosition.Y = 420;

            // 移動確定
            position = movePosition;
            // 方向確定
            Dir = dir;
        }

        // 購入
        public string Buy(Item item)
        {
            string message = "not enough money";

            // 金額チェック
            if (Money >= item.Price)
            {
                // 購入
                Money -= item.Price;

                // 装備
                switch (item.Type)
                {
                    case "Weapon":
                        Weapon = (Weapon)item;
                        break;
                    case "Armor":
                        Armor = (Armor)item;
                        break;
                }
                message = "You got " + item.Name + "!";
            }

            return message;
        }

        // 回復
        public string Heal(int fee)
        {
            string message = "not enough money";

            // 金額チェック
            if (Money >= fee)
            {
                // 支払
                Money -= fee;

                // 回復
                HP = MaxHP;
                Status = "Good";

                message = "You healed!";
            }

            return message;
        }

        // 戦う
        public string Attack(Character enemy)
        {
            string message = "miss";

            int atk = (int)((STR+Weapon.Value)*Game1.random.Next(50,100)/100-(enemy.DEF+enemy.Armor.Value)*Game1.random.Next(100)/100);
            if (atk > 0)
            {
                enemy.HP -= atk;
                message = Name + " attacked " + enemy.Name + "! [" + atk + "points]";
                if (enemy.HP <= 0) enemy.Status = "Dead";
            }
            return message;
        }

        // 勝利
        public string Victory(Character enemy)
        {
            string message;

            // EXP、Moneyの取得
            EXP += enemy.EXP;
            Money += enemy.Money;
            message = name + " won!  EXP: +" + enemy.EXP + ", Money: +" + enemy.Money;

            // レベルアップ
            if (EXP > Level * Level * 100)
            {
                int upHP = Game1.random.Next(50);
                int upSTR = Game1.random.Next(10);
                int upDEF = Game1.random.Next(10);
                Level++;
                MaxHP += upHP;
                STR += upSTR;
                DEF += upDEF;
                message += "\r\nLevel UP!!  HP: +" + upHP + ", STR: +" + upSTR + ", DEF: +" + upDEF;
            }
            return message;
        }

        // ステータス情報の表示
        public void DrawData(SpriteBatch spriteBatch,SpriteFont font)
        {
            Color statusColor = new Color(77,72,61);
            
            spriteBatch.DrawString(font, Name, new Vector2(630, 40), statusColor);

            spriteBatch.DrawString(font, "Status: " + Status, new Vector2(630, 80), statusColor);
            spriteBatch.DrawString(font, "HP: " + HP + "/" + MaxHP, new Vector2(630, 105), statusColor);
            spriteBatch.DrawString(font, "Level: " + Level, new Vector2(630, 130), statusColor);
            spriteBatch.DrawString(font, "EXP: " + EXP, new Vector2(630, 155), statusColor);
            spriteBatch.DrawString(font, "STR: " + STR + " (+" + Weapon.Value + ")", new Vector2(630, 180), statusColor);
            spriteBatch.DrawString(font, "DEF: " + DEF + " (+" + Armor.Value + ")", new Vector2(630, 205), statusColor);

            spriteBatch.DrawString(font, "Money: " + Money, new Vector2(630, 245), statusColor);

            spriteBatch.DrawString(font, "Weapon:", new Vector2(630, 285), statusColor);
            spriteBatch.DrawString(font, "[" + Weapon.Name + "]", new Vector2(630, 310), statusColor);
            spriteBatch.DrawString(font, "Armor:", new Vector2(630, 350), statusColor);
            spriteBatch.DrawString(font, "[" + Armor.Name + "]", new Vector2(630, 375), statusColor);
        }

        // キャラクタの表示（アニメーション）
        public void DrawCharacter(SpriteBatch spriteBatch, GameTime gameTime)
        {
            // フレームの指定（毎秒4フレーム、0～3フレームの繰り返し）
            frame += (float)gameTime.ElapsedGameTime.TotalSeconds * 4;
            if (frame > 4) frame = 0;

            spriteBatch.Draw(image, position, new Rectangle(60 * (int)frame, 90 * (int)Dir, 60, 90), Color.White, 0, origin, 1, SpriteEffects.None, position.Y / 450);
        }
    }
}
