﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace RPGTest
{
    // シーン列挙体
    public enum Scene
    {
        Map,
        Shop,
        Church,
        Battle
    }

    // コマンド列挙体
    public enum Command
    {
        Move,
        Restart,
        Quit,
        Weapon,
        Armor,
        Exit,
        Heal,
        Attack,
        Run
    }

    // 方向
    public enum Direction
    {
        Down = 0,
        Left,
        Right,
        Up
    }

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        // キーボード状態の宣言
        KeyboardState keyboardState;
        KeyboardState prevKeyboardState;
        // フォントの宣言
        SpriteFont font;
        // ベース画像の宣言
        Texture2D baseWin;
        // 乱数の宣言
        public static Random random = new Random();

        // マップオブジェクトの宣言
        MapObject weaponShop = new MapObject("Weapon Shop", "weaponshop", Scene.Shop, "shop_scene");
        MapObject church = new MapObject("Church", "church", Scene.Church, "church");
        MapObject enemyCastle = new MapObject("Enemy Castle", "enemycastle", Scene.Battle, "enemycastle");

        // マップの宣言
        Map map = new Map("Map", "map", "battle_scene");

        // 主人公、敵の宣言
        Character player = new Character();
        Character[] baseEnemy = new Character[4];
        Character enemy = new Character();

        // 武器、鎧の宣言
        Weapon[] weapon = new Weapon[8];
        Armor[] armor = new Armor[8];

        // シーンの宣言
        Scene scene = Scene.Map;

        // コマンドの宣言
        int commandNum = 4;
        int commandPos = 0;
        Command[] command = new Command[4];

        // 商品の宣言
        string shopItem = "Back";
        int shopItemPos = 0;

        // メッセージテキストの宣言
        string message = "";

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            // アイテムの初期化
            SetItems();

            // マップの初期化
            map.Clear();
            map.SetMapData(enemyCastle,new Rectangle(2,2,4,2));
            map.SetMapData(weaponShop,new Rectangle(14,4,4,2));
            map.SetMapData(church,new Rectangle(16,12,4,2));

            // 主人公の初期化
            player.SetCharacterData("Glasses", "players16", 100, 100, 10, 10, 1, 0, "Good", 100, weapon[0], armor[0], GetPosition(new Point(11, 8)));

            // 敵の初期化
            SetEnemies();

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            // フォントの呼出
            font = Content.Load<SpriteFont>("font");
            // ベース画像の呼出
            baseWin = Content.Load<Texture2D>("baseWin");

            // マップデータの呼出
            map.LoadMap(Content);

            // 主人公画像の呼出
            player.Load(Content);

            // 敵画像の呼出
            for (int i = 0; i < baseEnemy.Length; i++)
                baseEnemy[i].Load(Content);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            // キーボード状態の取得
            prevKeyboardState = keyboardState;
            keyboardState = Keyboard.GetState();
            if (keyboardState.IsKeyDown(Keys.Escape)) Exit();

            switch (scene)
            {
                // マップシーン
                case Scene.Map:
                    // コマンド設定
                    commandNum = 3;
                    command[0] = Command.Move;
                    command[1] = Command.Restart;
                    command[2] = Command.Quit;

                    switch (SelectCommand())
                    {
                        // 移動
                        case Command.Move:
                            if (keyboardState.IsKeyDown(Keys.Up))
                            {
                                player.Move(Direction.Up);
                                SetScene();
                            }
                            else if (keyboardState.IsKeyDown(Keys.Down))
                            {
                                player.Move(Direction.Down);
                                SetScene();
                            }
                            else if (keyboardState.IsKeyDown(Keys.Left))
                            {
                                player.Move(Direction.Left);
                                SetScene();
                            }
                            else if (keyboardState.IsKeyDown(Keys.Right))
                            {
                                player.Move(Direction.Right);
                                SetScene();
                            }
                            break;
                        // 再スタート
                        case Command.Restart:
                            CommandRestart();
                            break;
                        // 終了
                        case Command.Quit:
                            Exit();
                            break;
                    }
                    break;

                // ショップシーン
                case Scene.Shop:
                    // コマンド設定
                    commandNum = 3;
                    command[0] = Command.Weapon;
                    command[1] = Command.Armor;
                    command[2] = Command.Exit;
                    switch (shopItem)
                    {
                        case "Weapon":
                            SelectItem(weapon);
                            break;
                        case "Armor":
                            SelectItem(armor);
                            break;
                        case "Back":
                            switch (SelectCommand())
                            {
                                // 武器
                                case Command.Weapon:
                                    shopItem = Command.Weapon.ToString();
                                    break;
                                // 鎧
                                case Command.Armor:
                                    shopItem = Command.Armor.ToString();
                                    break;
                                // 出る
                                case Command.Exit:
                                    message = "Thanks.";
                                    CommandExit();
                                    break;
                            }
                            break;
                    }
                    break;

                // 教会シーン
                case Scene.Church:
                    // コマンド設定
                    commandNum = 2;
                    command[0] = Command.Heal;
                    command[1] = Command.Exit;
                    switch (SelectCommand())
                    {
                        // 治す
                        case Command.Heal:
                            message = player.Heal(100);
                            break;
                        // 出る
                        case Command.Exit:
                            CommandExit();
                            break;
                    }
                    break;

                // 戦闘シーン
                case Scene.Battle:
                    // コマンド設定
                    commandNum = 2;
                    command[0] = Command.Attack;
                    command[1] = Command.Run;
                    switch (SelectCommand())
                    {
                        // 戦う
                        case Command.Attack:
                            // 主人公の攻撃
                            message = player.Attack(enemy);
                            // 戦闘終了処理
                            if (enemy.Status == "Dead")
                            {
                                // 勝利
                                message += "\n" + player.Victory(enemy);
                                CommandExit();
                            }
                            else
                            {
                                // 敵の攻撃
                                message += "\n" + enemy.Attack(player);
                            }
                            if (player.Status == "Dead")
                            {
                                // GameOver
                                message += "\nGame Over";
                                CommandExit();
                            }
                            break;
                        // 逃げる
                        case Command.Run:
                            CommandExit();
                            break;
                    }
                    break;
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            //graphics.GraphicsDevice.Clear(Color.CornflowerBlue);
            graphics.GraphicsDevice.Clear(Color.DimGray);

            // TODO: Add your drawing code here
            Color baseColor = new Color(77, 72, 61);
            Color selectColor = Color.ForestGreen;
            
            // 主人公のマップ座標
            Point mapPosition = GetMapPosition(player.Position);

            // メインパネル
            switch (scene)
            {
                // マップシーン
                case Scene.Map:
                    spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.FrontToBack, SaveStateMode.None);
                    // マップデータの表示
                    map.DrawMap(spriteBatch);
                    // 主人公の表示
                    player.DrawCharacter(spriteBatch, gameTime);
                    spriteBatch.End();
                    break;

                // ショップシーン
                case Scene.Shop:
                    Color subCommandColor;

                    spriteBatch.Begin();
                    // シーン画像の表示
                    map[mapPosition.X, mapPosition.Y].MapObject.DrawScene(spriteBatch);

                    if (shopItem == "Weapon")
                    {
                        for (int i = 1; i < weapon.Length; i++)
                        {
                            subCommandColor = baseColor;
                            if (shopItemPos == i) subCommandColor = selectColor;
                            spriteBatch.DrawString(font, weapon[i].Name + " (" + weapon[i].Price + ")", new Vector2(30, 30 + i * 25), subCommandColor);
                        }
                        subCommandColor = baseColor;
                        if (shopItemPos == weapon.Length) subCommandColor = selectColor;
                        spriteBatch.DrawString(font, "Back", new Vector2(30, 30 + weapon.Length * 25), subCommandColor);
                    }
                    if (shopItem == "Armor")
                    {
                        for (int i = 1; i < armor.Length; i++)
                        {
                            subCommandColor = baseColor;
                            if (shopItemPos == i) subCommandColor = selectColor;
                            spriteBatch.DrawString(font, armor[i].Name + " (" + armor[i].Price + ")", new Vector2(30, 30 + i * 25), subCommandColor);
                        }
                        subCommandColor = baseColor;
                        if (shopItemPos == armor.Length) subCommandColor = selectColor;
                        spriteBatch.DrawString(font, "Back", new Vector2(30, 30 + armor.Length * 25), subCommandColor);
                    }

                    spriteBatch.End();
                    break;

                // 教会シーン
                case Scene.Church:
                    spriteBatch.Begin();
                    // シーン画像の表示
                    map[mapPosition.X, mapPosition.Y].MapObject.DrawScene(spriteBatch);
                    spriteBatch.End();
                    break;

                // 戦闘シーン
                case Scene.Battle:
                    spriteBatch.Begin();
                    // シーン画像の表示
                    map[mapPosition.X, mapPosition.Y].MapObject.DrawScene(spriteBatch);
                    enemy.setCenter();
                    enemy.Draw(spriteBatch);
                    spriteBatch.End();
                    break;
            }
            
            
            // ステータスパネル、コマンドパネル、メッセージパネル
            spriteBatch.Begin();
            // ベース画像
            spriteBatch.Draw(baseWin, Vector2.Zero, Color.White);

            // コマンド表示
            Color commandColor;
            for (int i = 0; i < commandNum; i++)
            {
                commandColor = baseColor;
                if (commandPos == i) commandColor = selectColor;
                spriteBatch.DrawString(font, command[i].ToString(), new Vector2(30, 480 + i * 25), commandColor);
            }

            // ステータス表示
            player.DrawData(spriteBatch, font);

            // メッセージ表示
            spriteBatch.DrawString(font, message, new Vector2(250, 480), baseColor);

            spriteBatch.End();

            base.Draw(gameTime);
        }

        // アイテムの初期化
        public void SetItems()
        {
            // 武器の初期化
            for (int i = 0; i < weapon.Length; i++) weapon[i] = new Weapon();
            weapon[0].SetData("no weapon", 0, 0);
            weapon[1].SetData("Chopsticks", 50, 10);
            weapon[2].SetData("Magazine", 100, 20);
            weapon[3].SetData("Slippers", 150, 30);
            weapon[4].SetData("Washbowl", 200, 40);
            weapon[5].SetData("Toy sword", 300, 50);
            weapon[6].SetData("Excalibur", 400, 60);
            weapon[7].SetData("Katana", 500, 70);

            // 鎧の初期化
            for (int i = 0; i < armor.Length; i++) armor[i] = new Armor();
            armor[0].SetData("no armor", 0, 0);
            armor[1].SetData("Clothes", 50, 10);
            armor[2].SetData("T-shirt", 100, 20);
            armor[3].SetData("Jacket", 150, 30);
            armor[4].SetData("Sweater", 200, 40);
            armor[5].SetData("Suit", 300, 50);
            armor[6].SetData("Tuxedo", 400, 60);
            armor[7].SetData("Yoroi", 500, 70);
        }

        // 敵の初期化
        public void SetEnemies()
        {
            for (int i = 0; i < baseEnemy.Length; i++) baseEnemy[i] = new Character();
            baseEnemy[0].SetCharacterData("Weak Enemy", "enemy1", 50, 50, 5, 5, 1, 50, "Good", 50, weapon[0], armor[0], Vector2.Zero);
            baseEnemy[1].SetCharacterData("Middle Enemy", "enemy1", 100, 100, 10, 10, 2, 100, "Good", 100, weapon[0], armor[0], Vector2.Zero);
            baseEnemy[2].SetCharacterData("Strong Enemy", "enemy1", 150, 150, 20, 20, 3, 150, "Good", 200, weapon[0], armor[0], Vector2.Zero);
            baseEnemy[3].SetCharacterData("Last Enemy", "enemy1", 300, 300, 50, 50, 6, 300, "Good", 300, weapon[0], armor[0], Vector2.Zero);
        }

        // 実座標からマップ座標を取得
        public Point GetMapPosition(Vector2 position)
        {
            Point mapPosition = Point.Zero;
            mapPosition.X = (int)position.X / 30;
            mapPosition.Y = (int)position.Y / 30;
            return mapPosition;
        }

        // マップ座標から実座標を取得
        public Vector2 GetPosition(Point mapPosition)
        {
            Vector2 position = Vector2.Zero;
            position.X = mapPosition.X * 30;
            position.Y = mapPosition.Y * 30;
            return position;
        }

        // シーンの設定
        public void SetScene()
        {
            // 主人公の位置からシーンを取得
            Point mapPosition = GetMapPosition(player.Position);
            scene = map[mapPosition.X, mapPosition.Y].MapObject.Scene;

            string temp;

            // ラスボス
            if (scene == Scene.Battle)
            {
                enemy = baseEnemy[3];
                // 敵の回復
                temp = enemy.Heal(0);
                message = enemy.Name + " showed up!";
            }
            // ランダム（100回に1回）で戦闘シーン
            if ((scene == Scene.Map) && (random.Next(100) == 1))
            {
                // 主人公のレベルにあわせて敵のレベルも変える
                if (player.Level < 3)
                {
                    enemy = baseEnemy[random.Next(player.Level)];
                }
                else
                {
                    enemy = baseEnemy[2];
                }
                scene = Scene.Battle;
                // 敵の回復
                temp = enemy.Heal(0);
                message = enemy.Name + " showed up!";
            }
            // ショップ
            if (scene == Scene.Shop) message = "May I help you?";
        }

        // コマンドを選択
        public Command SelectCommand()
        {
            // コマンド選択
            if ((keyboardState.IsKeyDown(Keys.X)) && (prevKeyboardState.IsKeyUp(Keys.X))) commandPos++;
            if ((keyboardState.IsKeyDown(Keys.Z)) && (prevKeyboardState.IsKeyUp(Keys.Z))) commandPos--;
            if (commandPos >= commandNum) commandPos = 0;
            if (commandPos < 0) commandPos = commandNum - 1;

            // コマンド確定
            if ((keyboardState.IsKeyDown(Keys.Space)) && (prevKeyboardState.IsKeyUp(Keys.Space)))
            {
                return command[commandPos];
            }
            else
            {
                return Command.Move;
            }
        }

        // アイテムを選択
        public void SelectItem(Item[] item)
        {
            // アイテム選択
            if ((keyboardState.IsKeyDown(Keys.X)) && (prevKeyboardState.IsKeyUp(Keys.X))) shopItemPos++;
            if ((keyboardState.IsKeyDown(Keys.Z)) && (prevKeyboardState.IsKeyUp(Keys.Z))) shopItemPos--;
            if (shopItemPos > item.Length) shopItemPos = 1;
            if (shopItemPos < 1) shopItemPos = item.Length;

            // アイテム確定
            if ((keyboardState.IsKeyDown(Keys.Space)) && (prevKeyboardState.IsKeyUp(Keys.Space)))
            {
                if (shopItemPos < item.Length)
                {
                    // 購入
                    player.Buy(item[shopItemPos]);
                }
                else
                {
                    // 戻る
                    shopItem = "Back";
                    shopItemPos = 0;
                }
            } 
        }
       
        // 建物から出る
        public void CommandExit()
        {
            Point mapPosition = GetMapPosition(player.Position);

            if (map[mapPosition.X, mapPosition.Y].MapObject.Scene != Scene.Map)
                player.Position = map[mapPosition.X, mapPosition.Y].OutPosition;
            scene = Scene.Map;
            commandPos = 0;
            player.Dir = Direction.Down;
        }

        // 再スタート
        public void CommandRestart()
        {
            player.SetCharacterData("Glasses", "players16", 100, 100, 10, 10, 1, 0, "Good", 100, weapon[0], armor[0], GetPosition(new Point(11, 8)));
            scene = Scene.Map;
            commandPos = 0;
            player.Dir = Direction.Down;
        }
    }
}
