﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace RPGTest
{
    // マップクラス（マップオブジェクトクラスを継承）
    class Map : MapObject
    {
        // マップデータ構造体
        public struct MapData
        {
            public MapObject MapObject;
            public Vector2 OutPosition;
        }

        // マップデータ（20×15）
        MapData[,] mapData = new MapData[20, 15];
        public MapData this[int x, int y]
        {
            set
            {
                mapData[x, y] = value;
            }
            get
            {
                return mapData[x, y];
            }
        }

        // マップオブジェクトリスト
        List<MapObject> mapObjectList = new List<MapObject>();

        // マップコンストラクタ
        public Map(string name, string asset, string sceneAsset)
        {
            // 初期化
            this.name = name;
            this.asset = asset;
            scene = Scene.Map;
            this.sceneAsset = sceneAsset;
            origin = new Vector2(0, 0);
        }

        // マップデータの初期化
        public void Clear()
        {
            // マップ座標のクリア
            for (int x = 0; x < 20; x++)
                for (int y = 0; y < 15; y++)
                {
                    mapData[x, y].MapObject = this;
                    mapData[x, y].OutPosition = Vector2.Zero;
                }
            // マップオブジェクトのクリア
            mapObjectList.Clear();
            // マップ自身を登録
            mapObjectList.Add(this);
        }

        // マップデータの作成
        public void SetMapData(MapObject mapObject, Rectangle position)
        {
            // マップ座標の作成
            for (int x = position.X; x < position.X + position.Width; x++)
                for (int y = position.Y; y < position.Y + position.Height; y++)
                {
                    mapData[x, y].MapObject = mapObject;
                    mapData[x, y].OutPosition = new Vector2(position.X * 30 + 60, position.Y * 30 + 60);
                }
            // マップオブジェクトの配置位置を決定
            mapObject.Position = new Vector2(position.X * 30, position.Y * 30);
            // マップオブジェクトの登録
            mapObjectList.Add(mapObject);
        }

        // マップオブジェクトの一括呼出
        public void LoadMap(ContentManager content)
        {
            // マップオブジェクトの呼出
            foreach (MapObject mapObject in mapObjectList)
            {
                // マップオブジェクト画像の呼出
                mapObject.Load(content);
                // シーン画像の呼出
                mapObject.LoadScene(content);
            }
        }

        // マップオブジェクトの一括表示
        public void DrawMap(SpriteBatch spriteBatch)
        {
            // マップオブジェクトの表示
            foreach (MapObject mapObject in mapObjectList)
            {
                mapObject.Draw(spriteBatch);
            }
        }
    }
}
